@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015, 2019 Andrew M. Bishop
+ This file Copyright 2008-2015, 2019, 2020 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -103,6 +103,10 @@ NodesX *NewNodeList(int append,int readonly)
  log_malloc(nodesx->cache,sizeof(*nodesx->cache));
 #endif
 
+ nodesx->ifilename_tmp=(char*)malloc_logassert(strlen(option_tmpdirname)+40); /* allow %p to be up to 20 bytes */
+
+ sprintf(nodesx->ifilename_tmp,"%s/nodesx.%p.idx.tmp",option_tmpdirname,(void*)nodesx);
+
  return(nodesx);
 }
 
@@ -125,11 +129,9 @@ void FreeNodeList(NodesX *nodesx,int keep)
  free(nodesx->filename);
  free(nodesx->filename_tmp);
 
- if(nodesx->idata)
-   {
-    log_free(nodesx->idata);
-    free(nodesx->idata);
-   }
+ DeleteFile(nodesx->ifilename_tmp);
+
+ free(nodesx->ifilename_tmp);
 
  if(nodesx->gdata)
    {
@@ -176,7 +178,7 @@ void FreeNodeList(NodesX *nodesx,int keep)
 
 void AppendNodeList(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,nodeflags_t flags)
 {
- NodeX nodex;
+ NodeX nodex={0};
 
  nodex.id=id;
  nodex.latitude =radians_to_latlong(latitude);
@@ -224,35 +226,28 @@ index_t IndexNodeX(NodesX *nodesx,node_t id)
  if(nodesx->number==0)          /* No nodes */
     return(NO_NODE);
 
- if(id<nodesx->idata[start])    /* Key is before start */
-    return(NO_NODE);
-
- if(id>nodesx->idata[end])      /* Key is after end */
-    return(NO_NODE);
-
  /* Binary search - search key exact match only is required.
   *
-  *  # <- start  |  Check mid and move start or end if it doesn't match
+  *  # <- start  |  Check mid and exit if it matches else move start or end.
   *  #           |
   *  #           |  Since an exact match is wanted we can set end=mid-1
-  *  # <- mid    |  or start=mid+1 because we know that mid doesn't match.
+  *  # <- mid    |  or start=mid+1 if we find that mid doesn't match.
   *  #           |
   *  #           |  Eventually either end=start or end=start+1 and one of
-  *  # <- end    |  start or end is the wanted one.
+  *  # <- end    |  start or end is the wanted one or neither is.
   */
 
- do
+ while((end-start)>1)
    {
     mid=start+(end-start)/2;       /* Choose mid point (avoid overflow) */
 
     if(nodesx->idata[mid]<id)      /* Mid point is too low */
        start=mid+1;
     else if(nodesx->idata[mid]>id) /* Mid point is too high */
-       end=mid?(mid-1):mid;
+       end=mid-1;
     else                           /* Mid point is correct */
        return(mid);
    }
- while((end-start)>1);
 
  if(nodesx->idata[start]==id)      /* Start is correct */
     return(start);
@@ -283,10 +278,9 @@ void SortNodeList(NodesX *nodesx)
 
  fd=ReplaceFileBuffered(nodesx->filename_tmp,&nodesx->fd);
 
- /* Allocate the array of indexes */
+ /* Open a file for the index */
 
- nodesx->idata=(node_t*)malloc_logassert(nodesx->number*sizeof(node_t));
- log_malloc(nodesx->idata,nodesx->number*sizeof(node_t));
+ nodesx->ifd=OpenFileBufferedNew(nodesx->ifilename_tmp);
 
  /* Sort the nodes by ID and index them */
 
@@ -305,6 +299,8 @@ void SortNodeList(NodesX *nodesx)
  nodesx->fd=CloseFileBuffered(nodesx->fd);
  CloseFileBuffered(fd);
 
+ nodesx->ifd=CloseFileBuffered(nodesx->ifd);
+
  /* Print the final message */
 
  printf_last("Sorted Nodes: Nodes=%"Pindex_t" Duplicates=%"Pindex_t,xnumber,xnumber-nodesx->number);
@@ -357,7 +353,7 @@ static int deduplicate_and_index_by_id(NodeX *nodex,index_t index)
        return(0);
     else
       {
-       sortnodesx->idata[index]=nodex->id;
+       WriteFileBuffered(sortnodesx->ifd,&nodex->id,sizeof(node_t));
 
        return(1);
       }
@@ -397,6 +393,10 @@ void RemoveNonHighwayNodes(NodesX *nodesx,WaysX *waysx,int keep)
 
  waysx->fd=ReOpenFileBuffered(waysx->filename_tmp);
 
+ /* Map the index into memory */
+
+ nodesx->idata=MapFile(nodesx->ifilename_tmp);
+
  /* Loop through the ways and mark the used nodes */
 
  for(i=0;i<waysx->number;i++)
@@ -432,16 +432,14 @@ void RemoveNonHighwayNodes(NodesX *nodesx,WaysX *waysx,int keep)
        printf_middle("Checking Ways for unused Nodes: Ways=%"Pindex_t" Highway Nodes=%"Pindex_t,i+1,highway);
    }
 
- /* Free the now-unneeded index */
-
- log_free(nodesx->idata);
- free(nodesx->idata);
- nodesx->idata=NULL;
-
  /* Close the file */
 
  waysx->fd=CloseFileBuffered(waysx->fd);
 
+ /* Unmap the index from memory */
+
+ nodesx->idata=UnmapFile(nodesx->idata);
+
  /* Print the final message */
 
  printf_last("Checked Ways for unused Nodes: Ways=%"Pindex_t" Highway Nodes=%"Pindex_t,waysx->number,highway);
@@ -451,10 +449,9 @@ void RemoveNonHighwayNodes(NodesX *nodesx,WaysX *waysx,int keep)
 
  printf_first("Removing unused Nodes: Nodes=0");
 
- /* Allocate the array of indexes */
+ /* Open a file for the index */
 
- nodesx->idata=(node_t*)malloc_logassert(highway*sizeof(node_t));
- log_malloc(nodesx->idata,highway*sizeof(node_t));
+ nodesx->ifd=OpenFileBufferedNew(nodesx->ifilename_tmp);
 
  highway=0;
 
@@ -479,10 +476,10 @@ void RemoveNonHighwayNodes(NodesX *nodesx,WaysX *waysx,int keep)
        nothighway++;
     else
       {
-       nodesx->idata[highway]=nodex.id;
-
        WriteFileBuffered(fd,&nodex,sizeof(NodeX));
 
+       WriteFileBuffered(nodesx->ifd,&nodex.id,sizeof(node_t));
+
        highway++;
       }
 
@@ -499,6 +496,8 @@ void RemoveNonHighwayNodes(NodesX *nodesx,WaysX *waysx,int keep)
  nodesx->fd=CloseFileBuffered(nodesx->fd);
  CloseFileBuffered(fd);
 
+ nodesx->ifd=CloseFileBuffered(nodesx->ifd);
+
  /* Free the now-unneeded index */
 
  log_free(usednode);
