@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2012 Andrew M. Bishop
+ This file Copyright 2008-2012, 2018, 2019 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -28,16 +28,35 @@
 #include "types.h"
 
 
-/* Functions in sorting.c */
+/* Constants */
 
 /*+ The type, size and alignment of variable to store the variable length +*/
-#define FILESORT_VARINT   unsigned short
+
+#define FILESORT_VARINT   uint16_t
+
 #define FILESORT_VARSIZE  sizeof(FILESORT_VARINT)
+
+#define FILESORT_MAXINT   ((FILESORT_VARINT)~0)
+
 #define FILESORT_VARALIGN sizeof(void*)
 
-index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*compare)(const void*,const void*),int (*keep)(void*,index_t));
 
-index_t filesort_vary(int fd_in,int fd_out,int (*compare)(const void*,const void*),int (*keep)(void*,index_t));
+/* Macros */
+
+/*+ A macro to use as a last resort in the comparison function to preserve
+    on the output the input order of items that compare equally. +*/
+#define FILESORT_PRESERVE_ORDER(a,b) ( ((a)<(b)) ? -1 : +1)
+
+
+/* Functions in sorting.c */
+
+index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_function)(void*,index_t),
+                                                            int (*compare_function)(const void*,const void*),
+                                                            int (*post_sort_function)(void*,index_t));
+
+index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_t),
+                                           int (*compare_function)(const void*,const void*),
+                                           int (*post_sort_function)(void*,index_t));
 
 void filesort_heapsort(void **datap,size_t nitems,int(*compare)(const void*, const void*));
 
