@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2014 Andrew M. Bishop
+ This file Copyright 2010-2015 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -44,8 +44,14 @@
 #define TAGACTION_OUTPUT   6
 #define TAGACTION_LOGERROR 7
 
+static const char* const default_logerror_message="ignoring it";
 
-/* Local variables */
+
+/* Local variable (intialised before each use) */
+
+static int64_t current_id;
+
+/* Local parsing variables (re-initialised by DeleteXMLTaggingRules() function) */
 
 static TaggingRuleList NodeRules={NULL,0};
 static TaggingRuleList WayRules={NULL,0};
@@ -55,12 +61,7 @@ static int current_list_stack_depth=0;
 static TaggingRuleList **current_list_stack=NULL;
 static TaggingRuleList *current_list=NULL;
 
-static int64_t current_id;
-
-static char *default_logerror_message="ignoring it";
-
-
-/* Local functions */
+/* Local parsing functions */
 
 static TaggingRuleList *AppendTaggingRule(TaggingRuleList *rules,const char *k,const char *v,int action);
 static void AppendTaggingAction(TaggingRuleList *rules,const char *k,const char *v,int action,const char *message);
@@ -86,96 +87,96 @@ static int LogErrorType_function(const char *_tag_,int _type_,const char *k,cons
 
 /* The XML tag definitions (forward declarations) */
 
-static xmltag xmlDeclaration_tag;
-static xmltag RoutinoTaggingType_tag;
-static xmltag NodeType_tag;
-static xmltag WayType_tag;
-static xmltag RelationType_tag;
-static xmltag IfType_tag;
-static xmltag IfNotType_tag;
-static xmltag SetType_tag;
-static xmltag UnsetType_tag;
-static xmltag OutputType_tag;
-static xmltag LogErrorType_tag;
+static const xmltag xmlDeclaration_tag;
+static const xmltag RoutinoTaggingType_tag;
+static const xmltag NodeType_tag;
+static const xmltag WayType_tag;
+static const xmltag RelationType_tag;
+static const xmltag IfType_tag;
+static const xmltag IfNotType_tag;
+static const xmltag SetType_tag;
+static const xmltag UnsetType_tag;
+static const xmltag OutputType_tag;
+static const xmltag LogErrorType_tag;
 
 
 /* The XML tag definition values */
 
 /*+ The complete set of tags at the top level. +*/
-static xmltag *xml_toplevel_tags[]={&xmlDeclaration_tag,&RoutinoTaggingType_tag,NULL};
+static const xmltag * const xml_toplevel_tags[]={&xmlDeclaration_tag,&RoutinoTaggingType_tag,NULL};
 
 /*+ The xmlDeclaration type tag. +*/
-static xmltag xmlDeclaration_tag=
+static const xmltag xmlDeclaration_tag=
               {"xml",
                2, {"version","encoding"},
                NULL,
                {NULL}};
 
 /*+ The RoutinoTaggingType type tag. +*/
-static xmltag RoutinoTaggingType_tag=
+static const xmltag RoutinoTaggingType_tag=
               {"routino-tagging",
                0, {NULL},
                NULL,
                {&NodeType_tag,&WayType_tag,&RelationType_tag,NULL}};
 
 /*+ The NodeType type tag. +*/
-static xmltag NodeType_tag=
+static const xmltag NodeType_tag=
               {"node",
                0, {NULL},
                NodeType_function,
                {&IfType_tag,&IfNotType_tag,NULL}};
 
 /*+ The WayType type tag. +*/
-static xmltag WayType_tag=
+static const xmltag WayType_tag=
               {"way",
                0, {NULL},
                WayType_function,
                {&IfType_tag,&IfNotType_tag,NULL}};
 
 /*+ The RelationType type tag. +*/
-static xmltag RelationType_tag=
+static const xmltag RelationType_tag=
               {"relation",
                0, {NULL},
                RelationType_function,
                {&IfType_tag,&IfNotType_tag,NULL}};
 
 /*+ The IfType type tag. +*/
-static xmltag IfType_tag=
+static const xmltag IfType_tag=
               {"if",
                2, {"k","v"},
                IfType_function,
                {&IfType_tag,&IfNotType_tag,&SetType_tag,&UnsetType_tag,&OutputType_tag,&LogErrorType_tag,NULL}};
 
 /*+ The IfNotType type tag. +*/
-static xmltag IfNotType_tag=
+static const xmltag IfNotType_tag=
               {"ifnot",
                2, {"k","v"},
                IfNotType_function,
                {&IfType_tag,&IfNotType_tag,&SetType_tag,&UnsetType_tag,&OutputType_tag,&LogErrorType_tag,NULL}};
 
 /*+ The SetType type tag. +*/
-static xmltag SetType_tag=
+static const xmltag SetType_tag=
               {"set",
                2, {"k","v"},
                SetType_function,
                {NULL}};
 
 /*+ The UnsetType type tag. +*/
-static xmltag UnsetType_tag=
+static const xmltag UnsetType_tag=
               {"unset",
                1, {"k"},
                UnsetType_function,
                {NULL}};
 
 /*+ The OutputType type tag. +*/
-static xmltag OutputType_tag=
+static const xmltag OutputType_tag=
               {"output",
                2, {"k","v"},
                OutputType_function,
                {NULL}};
 
 /*+ The LogErrorType type tag. +*/
-static xmltag LogErrorType_tag=
+static const xmltag LogErrorType_tag=
               {"logerror",
                3, {"k","v","message"},
                LogErrorType_function,
@@ -466,6 +467,8 @@ int ParseXMLTaggingRules(const char *filename)
 
  fd=OpenFile(filename);
 
+ /* Initialise variables used for parsing */
+
  retval=ParseXML(fd,xml_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_ERRNONAME);
 
  CloseFile(fd);
@@ -486,6 +489,10 @@ int ParseXMLTaggingRules(const char *filename)
 
 void DeleteXMLTaggingRules(void)
 {
+ current_list_stack_depth=0;
+ current_list_stack=NULL;
+ current_list=NULL;
+
  DeleteTaggingRuleList(&NodeRules);
  DeleteTaggingRuleList(&WayRules);
  DeleteTaggingRuleList(&RelationRules);
@@ -569,7 +576,7 @@ static void AppendTaggingAction(TaggingRuleList *rules,const char *k,const char
  if(message)
     rules->rules[rules->nrules-1].message=strcpy(malloc(strlen(message)+1),message);
  else
-    rules->rules[rules->nrules-1].message=default_logerror_message;
+    rules->rules[rules->nrules-1].message=(char*)default_logerror_message;
 
  rules->rules[rules->nrules-1].rulelist=NULL;
 }
@@ -603,6 +610,9 @@ void DeleteTaggingRuleList(TaggingRuleList *rules)
 
  if(rules->rules)
     free(rules->rules);
+
+ rules->rules=NULL;
+ rules->nrules=0;
 }
 
 
@@ -740,7 +750,7 @@ void DeleteTag(TagList *tags,const char *k)
 
 char *StringifyTag(TagList *tags)
 {
- static char *string=NULL;
+ static char *string=NULL; /* static allocation of return value */
  int i,length=0,used=0;
 
  for(i=0;i<tags->ntags;i++)
