@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2015 Andrew M. Bishop
+ This file Copyright 2008-2015, 2018, 2019, 2020 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -73,12 +73,10 @@ WaysX *NewWayList(int append,int readonly)
 {
  WaysX *waysx;
 
- waysx=(WaysX*)calloc(1,sizeof(WaysX));
+ waysx=(WaysX*)calloc_logassert(1,sizeof(WaysX));
 
- logassert(waysx,"Failed to allocate memory (try using slim mode?)"); /* Check calloc() worked */
-
- waysx->filename    =(char*)malloc(strlen(option_tmpdirname)+32);
- waysx->filename_tmp=(char*)malloc(strlen(option_tmpdirname)+32); /* allow %p to be up to 20 bytes */
+ waysx->filename    =(char*)malloc_logassert(strlen(option_tmpdirname)+32);
+ waysx->filename_tmp=(char*)malloc_logassert(strlen(option_tmpdirname)+32); /* allow %p to be up to 20 bytes */
 
  sprintf(waysx->filename    ,"%s/waysx.parsed.mem",option_tmpdirname);
  sprintf(waysx->filename_tmp,"%s/waysx.%p.tmp"    ,option_tmpdirname,(void*)waysx);
@@ -115,8 +113,13 @@ WaysX *NewWayList(int append,int readonly)
  log_malloc(waysx->cache,sizeof(*waysx->cache));
 #endif
 
+ waysx->ifilename_tmp=(char*)malloc_logassert(strlen(option_tmpdirname)+40); /* allow %p to be up to 20 bytes */
+ waysx->ofilename_tmp=(char*)malloc_logassert(strlen(option_tmpdirname)+40); /* allow %p to be up to 20 bytes */
 
- waysx->nfilename_tmp=(char*)malloc(strlen(option_tmpdirname)+40); /* allow %p to be up to 20 bytes */
+ sprintf(waysx->ifilename_tmp,"%s/waysx.%p.idx.tmp",option_tmpdirname,(void*)waysx);
+ sprintf(waysx->ofilename_tmp,"%s/waysx.%p.off.tmp",option_tmpdirname,(void*)waysx);
+
+ waysx->nfilename_tmp=(char*)malloc_logassert(strlen(option_tmpdirname)+40); /* allow %p to be up to 20 bytes */
 
  sprintf(waysx->nfilename_tmp,"%s/waynames.%p.tmp",option_tmpdirname,(void*)waysx);
 
@@ -142,17 +145,11 @@ void FreeWayList(WaysX *waysx,int keep)
  free(waysx->filename);
  free(waysx->filename_tmp);
 
- if(waysx->idata)
-   {
-    log_free(waysx->idata);
-    free(waysx->idata);
-   }
+ DeleteFile(waysx->ifilename_tmp);
 
- if(waysx->odata)
-   {
-    log_free(waysx->odata);
-    free(waysx->odata);
-   }
+ DeleteFile(waysx->ofilename_tmp);
+
+ free(waysx->ifilename_tmp);
 
  if(waysx->cdata)
    {
@@ -191,14 +188,19 @@ void FreeWayList(WaysX *waysx,int keep)
 
 void AppendWayList(WaysX *waysx,way_t id,Way *way,node_t *nodes,int nnodes,const char *name)
 {
- WayX wayx;
+ WayX wayx={0};
+ uint64_t longsize;
  FILESORT_VARINT size;
  node_t nonode=NO_NODE_ID;
 
  wayx.id=id;
  wayx.way=*way;
 
- size=sizeof(WayX)+(nnodes+1)*sizeof(node_t)+strlen(name)+1;
+ longsize=sizeof(WayX)+(nnodes+1)*sizeof(node_t)+strlen(name)+1;
+
+ logassert(longsize<FILESORT_MAXINT,"Way contains too much data (change FILESORT_VARINT to 32-bits?)"); /* Ensure no overflow of FILESORT_VARINT integer */
+
+ size=longsize;
 
  WriteFileBuffered(waysx->fd,&size,FILESORT_VARSIZE);
  WriteFileBuffered(waysx->fd,&wayx,sizeof(WayX));
@@ -246,35 +248,28 @@ index_t IndexWayX(WaysX *waysx,way_t id)
  if(waysx->number==0)           /* There are no ways */
     return(NO_WAY);
 
- if(id<waysx->idata[start])     /* Key is before start */
-    return(NO_WAY);
-
- if(id>waysx->idata[end])       /* Key is after end */
-    return(NO_WAY);
-
  /* Binary search - search key exact match only is required.
   *
-  *  # <- start  |  Check mid and move start or end if it doesn't match
+  *  # <- start  |  Check mid and exit if it matches else move start or end.
   *  #           |
   *  #           |  Since an exact match is wanted we can set end=mid-1
-  *  # <- mid    |  or start=mid+1 because we know that mid doesn't match.
+  *  # <- mid    |  or start=mid+1 if we find that mid doesn't match.
   *  #           |
   *  #           |  Eventually either end=start or end=start+1 and one of
-  *  # <- end    |  start or end is the wanted one.
+  *  # <- end    |  start or end is the wanted one or neither is.
   */
 
- do
+ while((end-start)>1)
    {
-    mid=(start+end)/2;            /* Choose mid point */
+    mid=start+(end-start)/2;      /* Choose mid point (avoid overflow) */
 
     if(waysx->idata[mid]<id)      /* Mid point is too low */
        start=mid+1;
     else if(waysx->idata[mid]>id) /* Mid point is too high */
-       end=mid?(mid-1):mid;
+       end=mid-1;
     else                          /* Mid point is correct */
        return(mid);
    }
- while((end-start)>1);
 
  if(waysx->idata[start]==id)      /* Start is correct */
     return(start);
@@ -305,12 +300,9 @@ void SortWayList(WaysX *waysx)
 
  fd=ReplaceFileBuffered(waysx->filename_tmp,&waysx->fd);
 
- /* Allocate the array of indexes */
+ /* Open a file for the index */
 
- waysx->idata=(way_t*)malloc(waysx->number*sizeof(way_t));
- log_malloc(waysx->idata,waysx->number*sizeof(way_t));
-
- logassert(waysx->idata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
+ waysx->ifd=OpenFileBufferedNew(waysx->ifilename_tmp);
 
  /* Sort the ways by ID and index them */
 
@@ -329,6 +321,8 @@ void SortWayList(WaysX *waysx)
  waysx->fd=CloseFileBuffered(waysx->fd);
  CloseFileBuffered(fd);
 
+ waysx->ifd=CloseFileBuffered(waysx->ifd);
+
  /* Print the final message */
 
  printf_last("Sorted Ways: Ways=%"Pindex_t" Duplicates=%"Pindex_t,xnumber,xnumber-waysx->number);
@@ -381,7 +375,7 @@ static int deduplicate_and_index_by_id(WayX *wayx,index_t index)
        return(0);
     else
       {
-       sortwaysx->idata[index]=wayx->id;
+       WriteFileBuffered(sortwaysx->ifd,&wayx->id,sizeof(way_t));
 
        return(1);
       }
@@ -409,7 +403,7 @@ SegmentsX *SplitWays(WaysX *waysx,NodesX *nodesx,int keep)
  index_t i;
  int fd,nfd;
  char *name=NULL;
- int namelen=0;
+ uint32_t namelen=0;
 
  /* Print the start message */
 
@@ -432,6 +426,11 @@ SegmentsX *SplitWays(WaysX *waysx,NodesX *nodesx,int keep)
 
  nfd=OpenFileBufferedNew(waysx->nfilename_tmp);
 
+ /* Map the index into memory */
+
+ nodesx->idata=MapFile(nodesx->ifilename_tmp);
+ waysx->idata =MapFile(waysx->ifilename_tmp);
+
  /* Loop through the ways and create the segments and way names */
 
  for(i=0;i<waysx->number;i++)
@@ -445,7 +444,7 @@ SegmentsX *SplitWays(WaysX *waysx,NodesX *nodesx,int keep)
 
     ReadFileBuffered(waysx->fd,&wayx,sizeof(WayX));
 
-    waysx->allow|=wayx.way.allow;
+    waysx->transports|=wayx.way.allow;
 
     while(!ReadFileBuffered(waysx->fd,&node,sizeof(node_t)) && node!=NO_NODE_ID)
       {
@@ -483,7 +482,7 @@ SegmentsX *SplitWays(WaysX *waysx,NodesX *nodesx,int keep)
     size-=sizeof(node_t)+sizeof(WayX);
 
     if(namelen<size)
-       name=(char*)realloc((void*)name,namelen=size);
+       name=(char*)realloc_logassert((void*)name,namelen=size);
 
     ReadFileBuffered(waysx->fd,name,size);
 
@@ -510,6 +509,11 @@ SegmentsX *SplitWays(WaysX *waysx,NodesX *nodesx,int keep)
 
  CloseFileBuffered(nfd);
 
+ /* Unmap the index from memory */
+
+ nodesx->idata=UnmapFile(nodesx->idata);
+ waysx->idata =UnmapFile(waysx->idata);
+
  /* Print the final message */
 
  printf_last("Split Ways: Ways=%"Pindex_t" Segments=%"Pindex_t,waysx->number,segmentsx->number);
@@ -530,7 +534,7 @@ void SortWayNames(WaysX *waysx)
  index_t i;
  int nfd;
  char *names[2]={NULL,NULL};
- int namelen[2]={0,0};
+ uint32_t namelen[2]={0,0};
  int nnames=0;
  uint32_t lastlength=0;
 
@@ -564,7 +568,7 @@ void SortWayNames(WaysX *waysx)
 
  printf_first("Updating Ways with Names: Ways=0 Names=0");
 
- /* Map into memory /  open the file */
+ /* Map into memory / open the file */
 
 #if !SLIM
  waysx->data=MapFileWriteable(waysx->filename_tmp);
@@ -587,7 +591,7 @@ void SortWayNames(WaysX *waysx)
     ReadFileBuffered(waysx->nfd,&size,FILESORT_VARSIZE);
 
     if(namelen[nnames%2]<size)
-       names[nnames%2]=(char*)realloc((void*)names[nnames%2],namelen[nnames%2]=size);
+       names[nnames%2]=(char*)realloc_logassert((void*)names[nnames%2],namelen[nnames%2]=size);
 
     ReadFileBuffered(waysx->nfd,&index,sizeof(index_t));
     ReadFileBuffered(waysx->nfd,names[nnames%2],size-sizeof(index_t));
@@ -681,11 +685,9 @@ void CompactWayList(WaysX *waysx,SegmentsX *segmentsx)
 
  /* Allocate the array of indexes */
 
- waysx->cdata=(index_t*)malloc(waysx->number*sizeof(index_t));
+ waysx->cdata=(index_t*)malloc_logassert(waysx->number*sizeof(index_t));
  log_malloc(waysx->cdata,waysx->number*sizeof(index_t));
 
- logassert(waysx->cdata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
-
  /* Re-open the file read-only and a new file writeable */
 
  fd=ReplaceFileBuffered(waysx->filename_tmp,&waysx->fd);
@@ -794,8 +796,6 @@ static int deduplicate_and_index_by_compact_id(WayX *wayx,index_t index)
 
     sortwaysx->cdata[wayx->id]=index;
 
-    wayx->id=index;
-
     return(1);
    }
  else
@@ -823,8 +823,8 @@ void SaveWayList(WaysX *waysx,const char *filename)
  WayX wayx;
  WaysFile waysfile={0};
  highways_t   highways=0;
- transports_t allow=0;
- properties_t props=0;
+ transports_t transports=0;
+ properties_t properties=0;
 
  /* Print the start message */
 
@@ -845,9 +845,9 @@ void SaveWayList(WaysX *waysx,const char *filename)
    {
     ReadFileBuffered(waysx->fd,&wayx,sizeof(WayX));
 
-    highways|=HIGHWAYS(wayx.way.type);
-    allow   |=wayx.way.allow;
-    props   |=wayx.way.props;
+    highways  |=HIGHWAYS(wayx.way.type);
+    transports|=wayx.way.allow;
+    properties|=wayx.way.props;
 
     WriteFileBuffered(fd,&wayx.way,sizeof(Way));
 
@@ -883,9 +883,9 @@ void SaveWayList(WaysX *waysx,const char *filename)
 
  waysfile.number =waysx->number;
 
- waysfile.highways=highways;
- waysfile.allow   =allow;
- waysfile.props   =props;
+ waysfile.highways  =highways;
+ waysfile.transports=transports;
+ waysfile.properties=properties;
 
  SeekFileBuffered(fd,0);
  WriteFileBuffered(fd,&waysfile,sizeof(WaysFile));
