@@ -85,7 +85,7 @@ Select start and end points (click on the marker icons below), select routing pr
 <td>(EN)
 <td><input name="language" type="radio" value="en" onchange="formSetLanguage();" >
 <tr>
-<td><a id="lang_cs_url" href="router.html.cs" title="Česky webpage">Česky</a>
+<td><a id="lang_cs_url" href="router.html.cs" title="Stránka v češtině">Česky</a>
 <td>(CS)
 <td><input name="language" type="radio" value="cs" onchange="formSetLanguage();" checked>
 <tr>
@@ -97,6 +97,10 @@ Select start and end points (click on the marker icons below), select routing pr
 <td>(ES)
 <td><input name="language" type="radio" value="es" onchange="formSetLanguage();" >
 <tr>
+<td><a id="lang_fi_url" href="router.html.fi" title="Suomi webpage">Suomi</a>
+<td>(FI)
+<td><input name="language" type="radio" value="fi" onchange="formSetLanguage();" >
+<tr>
 <td><a id="lang_fr_url" href="router.html.fr" title="Page web en français">Français</a>
 <td>(FR)
 <td><input name="language" type="radio" value="fr" onchange="formSetLanguage();" >
@@ -117,11 +121,7 @@ Select start and end points (click on the marker icons below), select routing pr
 <td>(PL)
 <td><input name="language" type="radio" value="pl" onchange="formSetLanguage();" >
 <tr>
-<td><a id="lang_pt_url" href="router.html.pt" title="Português webpage">Português</a>
-<td>(PT)
-<td>
-<tr>
-<td><a id="lang_ru_url" href="router.html.ru" title="Страница на Русском языке">Русский</a>
+<td><a id="lang_ru_url" href="router.html.ru" title="Страница на русском языке">Русский</a>
 <td>(RU)
 <td><input name="language" type="radio" value="ru" onchange="formSetLanguage();" >
 </table>
@@ -153,9 +153,9 @@ Select start and end points (click on the marker icons below), select routing pr
 <br>
 <img alt="#" src="icons/waypoint-coords.png" title="Coordinates for location" onmousedown="markerCoords(XXX);" >
 <img alt="~" src="icons/waypoint-home.png" title="Přepnout jako domovské umístění" onmousedown="markerHome(XXX);" >
-<img alt="o" src="icons/waypoint-centre.png" title="Centre this waypoint on map" onmousedown="markerCentre(XXX);" >
-<img alt="v" src="icons/waypoint-down.png" title="Move this waypoint down" onmousedown="markerMoveDown(XXX);">
-<img alt="-" src="icons/waypoint-remove.png" title="Remove this waypoint" onmousedown="markerRemove(XXX);" >
+<img alt="o" src="icons/waypoint-centre.png" title="Vycentrovat tento bod na mapě" onmousedown="markerCentre(XXX);" >
+<img alt="v" src="icons/waypoint-down.png" title="Posunout tento bod níže" onmousedown="markerMoveDown(XXX);">
+<img alt="-" src="icons/waypoint-remove.png" title="Odstranit tento bod" onmousedown="markerRemove(XXX);" >
 </div>
 <div id="searchresultsXXX" style="display: none;">
 </div>
@@ -163,12 +163,12 @@ Select start and end points (click on the marker icons below), select routing pr
 </div>
 <div id="waypoints-buttons">
 <table>
-<tr><td>Close loop: <td><input type="checkbox" name="loop" onchange="formSetLoopReverse('loop' );">
-<tr><td>Reverse order:<td><input type="checkbox" name="reverse" onchange="formSetLoopReverse('reverse');">
+<tr><td>Uzavřít okruh: <td><input type="checkbox" name="loop" onchange="formSetLoopReverse('loop' );">
+<tr><td>Opačné pořadí:<td><input type="checkbox" name="reverse" onchange="formSetLoopReverse('reverse');">
 </table>
 <div class="waypoint-buttons" style="display: inline-block;">
-<img src="icons/waypoint-loop.png" title="Add a new waypoint to make a loop" onmousedown="markersLoop();">
-<img src="icons/waypoint-reverse.png" title="Reverse order of waypoints" onmousedown="markersReverse();">
+<img src="icons/waypoint-loop.png" title="Přidat bod pro vytvoření okruhu" onmousedown="markersLoop();">
+<img src="icons/waypoint-reverse.png" title="Opačné pořadí bodů" onmousedown="markersReverse();">
 </div>
 </div>
 </div>
@@ -193,7 +193,7 @@ Select start and end points (click on the marker icons below), select routing pr
 <tr><td>Moped: <td><input name="transport" type="radio" value="moped" onchange="formSetTransport('moped' );">
 <tr><td>Motocykl:<td><input name="transport" type="radio" value="motorcycle" onchange="formSetTransport('motorcycle');">
 <tr><td>Motorcar: <td><input name="transport" type="radio" value="motorcar" onchange="formSetTransport('motorcar' );">
-<tr><td>Goods: <td><input name="transport" type="radio" value="goods" onchange="formSetTransport('goods' );">
+<tr><td>Zboží: <td><input name="transport" type="radio" value="goods" onchange="formSetTransport('goods' );">
 <tr><td>HGV: <td><input name="transport" type="radio" value="hgv" onchange="formSetTransport('hgv' );">
 <tr><td>PSV: <td><input name="transport" type="radio" value="psv" onchange="formSetTransport('psv' );">
 </table>
@@ -217,8 +217,8 @@ Select start and end points (click on the marker icons below), select routing pr
 <tr><td>Track: <td><input name="highway-track" type="text" size="3" onchange="formSetHighway('track' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetHighway('track' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetHighway('track' ,'+');">
 <tr><td>Cycleway: <td><input name="highway-cycleway" type="text" size="3" onchange="formSetHighway('cycleway' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetHighway('cycleway' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetHighway('cycleway' ,'+');">
 <tr><td>Path: <td><input name="highway-path" type="text" size="3" onchange="formSetHighway('path' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetHighway('path' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetHighway('path' ,'+');">
-<tr><td>Steps: <td><input name="highway-steps" type="text" size="3" onchange="formSetHighway('steps' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetHighway('steps' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetHighway('steps' ,'+');">
-<tr><td>Ferry: <td><input name="highway-ferry" type="text" size="3" onchange="formSetHighway('ferry' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetHighway('ferry' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetHighway('ferry' ,'+');">
+<tr><td>Schody: <td><input name="highway-steps" type="text" size="3" onchange="formSetHighway('steps' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetHighway('steps' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetHighway('steps' ,'+');">
+<tr><td>Převoz: <td><input name="highway-ferry" type="text" size="3" onchange="formSetHighway('ferry' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetHighway('ferry' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetHighway('ferry' ,'+');">
 </table>
 </div>
 </div>
@@ -240,8 +240,8 @@ Select start and end points (click on the marker icons below), select routing pr
 <tr><td>Track: <td><input name="speed-track" type="text" size="3" onchange="formSetSpeed('track' ,'=');"><td>km/hr<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetSpeed('track' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetSpeed('track' ,'+');">
 <tr><td>Cycleway: <td><input name="speed-cycleway" type="text" size="3" onchange="formSetSpeed('cycleway' ,'=');"><td>km/hr<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetSpeed('cycleway' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetSpeed('cycleway' ,'+');">
 <tr><td>Path: <td><input name="speed-path" type="text" size="3" onchange="formSetSpeed('path' ,'=');"><td>km/hr<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetSpeed('path' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetSpeed('path' ,'+');">
-<tr><td>Steps: <td><input name="speed-steps" type="text" size="3" onchange="formSetSpeed('steps' ,'=');"><td>km/hr<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetSpeed('steps' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetSpeed('steps' ,'+');">
-<tr><td>Ferry: <td><input name="speed-ferry" type="text" size="3" onchange="formSetSpeed('ferry' ,'=');"><td>km/hr<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetSpeed('ferry' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetSpeed('ferry' ,'+');">
+<tr><td>Schody: <td><input name="speed-steps" type="text" size="3" onchange="formSetSpeed('steps' ,'=');"><td>km/hr<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetSpeed('steps' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetSpeed('steps' ,'+');">
+<tr><td>Převoz: <td><input name="speed-ferry" type="text" size="3" onchange="formSetSpeed('ferry' ,'=');"><td>km/hr<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetSpeed('ferry' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetSpeed('ferry' ,'+');">
 </table>
 </div>
 </div>
@@ -256,8 +256,8 @@ Select start and end points (click on the marker icons below), select routing pr
 <tr><td>Multiple Lanes: <td><input name="property-multilane" type="text" size="3" onchange="formSetProperty('multilane' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetProperty('multilane' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetProperty('multilane' ,'+');">
 <tr><td>Most: <td><input name="property-bridge" type="text" size="3" onchange="formSetProperty('bridge' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetProperty('bridge' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetProperty('bridge' ,'+');">
 <tr><td>Tunel: <td><input name="property-tunnel" type="text" size="3" onchange="formSetProperty('tunnel' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetProperty('tunnel' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetProperty('tunnel' ,'+');">
-<tr><td>Walking Route:<td><input name="property-footroute" type="text" size="3" onchange="formSetProperty('footroute' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetProperty('footroute' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetProperty('footroute' ,'+');">
-<tr><td>Bicycle Route:<td><input name="property-bicycleroute" type="text" size="3" onchange="formSetProperty('bicycleroute','=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetProperty('bicycleroute','-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetProperty('bicycleroute','+');">
+<tr><td>Pěší trasa:<td><input name="property-footroute" type="text" size="3" onchange="formSetProperty('footroute' ,'=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetProperty('footroute' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetProperty('footroute' ,'+');">
+<tr><td>Cyklotrasa:<td><input name="property-bicycleroute" type="text" size="3" onchange="formSetProperty('bicycleroute','=');"><td>%<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetProperty('bicycleroute','-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetProperty('bicycleroute','+');">
 </table>
 </div>
 </div>
@@ -268,14 +268,14 @@ Select start and end points (click on the marker icons below), select routing pr
 <span class="hideshow_title">Jiná omezení</span>
 <div id="hideshow_restriction_div" style="display: none;">
 <table>
-<tr><td>Obey oneway streets:<td><input name="restrict-oneway" type="checkbox" onchange="formSetRestriction('oneway');">
+<tr><td>Dodržovat jednosměrky:<td><input name="restrict-oneway" type="checkbox" onchange="formSetRestriction('oneway');">
 <tr><td>Obey turn restrictions: <td><input name="restrict-turns" type="checkbox" onchange="formSetRestriction('turns' );">
 </table>
 <table>
-<tr><td>Weight:<td><input name="restrict-weight" type="text" size="3" onchange="formSetRestriction('weight','=');"><td>tonnes<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetRestriction('weight','-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetRestriction('weight','+');">
-<tr><td>Height:<td><input name="restrict-height" type="text" size="3" onchange="formSetRestriction('height','=');"><td>metres<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetRestriction('height','-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetRestriction('height','+');">
-<tr><td>Width: <td><input name="restrict-width" type="text" size="3" onchange="formSetRestriction('width' ,'=');"><td>metres<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetRestriction('width' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetRestriction('width' ,'+');">
-<tr><td>Length:<td><input name="restrict-length" type="text" size="3" onchange="formSetRestriction('length','=');"><td>metres<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetRestriction('length','-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetRestriction('length','+');">
+<tr><td>Hmotnost:<td><input name="restrict-weight" type="text" size="3" onchange="formSetRestriction('weight','=');"><td>tonnes<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetRestriction('weight','-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetRestriction('weight','+');">
+<tr><td>Výška:<td><input name="restrict-height" type="text" size="3" onchange="formSetRestriction('height','=');"><td>metres<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetRestriction('height','-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetRestriction('height','+');">
+<tr><td>Šířka: <td><input name="restrict-width" type="text" size="3" onchange="formSetRestriction('width' ,'=');"><td>metres<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetRestriction('width' ,'-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetRestriction('width' ,'+');">
+<tr><td>Délka:<td><input name="restrict-length" type="text" size="3" onchange="formSetRestriction('length','=');"><td>metres<td><img alt="<" src="icons/waypoint-left.png" title="-" onmousedown="formSetRestriction('length','-');">&ndash;/+<img alt=">" src="icons/waypoint-right.png" title="+" onmousedown="formSetRestriction('length','+');">
 </table>
 </div>
 </div>
@@ -399,11 +399,11 @@ one-way restrictions (e.g. if walking).
 <div id="hideshow_shortest_div" style="display: none;">
 <div id="shortest_links" style="display: none;">
 <table>
-<tr><td>HTML directions: <td><a id="shortest_html" target="shortest_html" href="#">Open Popup</a>
-<tr><td>GPX track file: <td><a id="shortest_gpx_track" target="shortest_gpx_track" href="#">Open Popup</a>
-<tr><td>GPX route file: <td><a id="shortest_gpx_route" target="shortest_gpx_route" href="#">Open Popup</a>
-<tr><td>Full text file: <td><a id="shortest_text_all" target="shortest_text_all" href="#">Open Popup</a>
-<tr><td>Text file: <td><a id="shortest_text" target="shortest_text" href="#">Open Popup</a>
+<tr><td>HTML directions: <td><a id="shortest_html" target="shortest_html" href="#">Otevřít popup</a>
+<tr><td>GPX track file: <td><a id="shortest_gpx_track" target="shortest_gpx_track" href="#">Otevřít popup</a>
+<tr><td>GPX route file: <td><a id="shortest_gpx_route" target="shortest_gpx_route" href="#">Otevřít popup</a>
+<tr><td>Full text file: <td><a id="shortest_text_all" target="shortest_text_all" href="#">Otevřít popup</a>
+<tr><td>Textový soubor: <td><a id="shortest_text" target="shortest_text" href="#">Otevřít popup</a>
 </table>
 <hr>
 </div>
@@ -426,11 +426,11 @@ one-way restrictions (e.g. if walking).
 <div id="hideshow_quickest_div" style="display: none;">
 <div id="quickest_links" style="display: none;">
 <table>
-<tr><td>HTML directions: <td><a id="quickest_html" target="quickest_html" href="#">Open Popup</a>
-<tr><td>GPX track file: <td><a id="quickest_gpx_track" target="quickest_gpx_track" href="#">Open Popup</a>
-<tr><td>GPX route file: <td><a id="quickest_gpx_route" target="quickest_gpx_route" href="#">Open Popup</a>
-<tr><td>Full text file: <td><a id="quickest_text_all" target="quickest_text_all" href="#">Open Popup</a>
-<tr><td>Text file: <td><a id="quickest_text" target="quickest_text" href="#">Open Popup</a>
+<tr><td>HTML directions: <td><a id="quickest_html" target="quickest_html" href="#">Otevřít popup</a>
+<tr><td>GPX track file: <td><a id="quickest_gpx_track" target="quickest_gpx_track" href="#">Otevřít popup</a>
+<tr><td>GPX route file: <td><a id="quickest_gpx_route" target="quickest_gpx_route" href="#">Otevřít popup</a>
+<tr><td>Full text file: <td><a id="quickest_text_all" target="quickest_text_all" href="#">Otevřít popup</a>
+<tr><td>Textový soubor: <td><a id="quickest_text" target="quickest_text" href="#">Otevřít popup</a>
 </table>
 <hr>
 </div>
@@ -487,7 +487,7 @@ route.
 <div class="hideshow_box">
 <span class="hideshow_title">Routino statistiky</span>
 <div id="statistics_data"></div>
-<a id="statistics_link" href="statistics.cgi" onclick="displayStatistics();return(false);">Display data statistics</a>
+<a id="statistics_link" href="statistics.cgi" onclick="displayStatistics();return(false);">Zobrazit statistická data</a>
 </div>
 
 <div class="hideshow_box">
