@@ -85,7 +85,7 @@ Select start and end points (click on the marker icons below), select routing pr
 <td>(EN)
 <td><input name="language" type="radio" value="en" onchange="formSetLanguage();" >
 <tr>
-<td><a id="lang_cs_url" href="router.html.cs" title="Česky webpage">Česky</a>
+<td><a id="lang_cs_url" href="router.html.cs" title="Stránka v češtině">Česky</a>
 <td>(CS)
 <td><input name="language" type="radio" value="cs" onchange="formSetLanguage();" >
 <tr>
@@ -97,6 +97,10 @@ Select start and end points (click on the marker icons below), select routing pr
 <td>(ES)
 <td><input name="language" type="radio" value="es" onchange="formSetLanguage();" >
 <tr>
+<td><a id="lang_fi_url" href="router.html.fi" title="Suomi webpage">Suomi</a>
+<td>(FI)
+<td><input name="language" type="radio" value="fi" onchange="formSetLanguage();" >
+<tr>
 <td><a id="lang_fr_url" href="router.html.fr" title="Page web en français">Français</a>
 <td>(FR)
 <td><input name="language" type="radio" value="fr" onchange="formSetLanguage();" >
@@ -117,11 +121,7 @@ Select start and end points (click on the marker icons below), select routing pr
 <td>(PL)
 <td><input name="language" type="radio" value="pl" onchange="formSetLanguage();" checked>
 <tr>
-<td><a id="lang_pt_url" href="router.html.pt" title="Português webpage">Português</a>
-<td>(PT)
-<td>
-<tr>
-<td><a id="lang_ru_url" href="router.html.ru" title="Страница на Русском языке">Русский</a>
+<td><a id="lang_ru_url" href="router.html.ru" title="Страница на русском языке">Русский</a>
 <td>(RU)
 <td><input name="language" type="radio" value="ru" onchange="formSetLanguage();" >
 </table>
