@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2019 Andrew M. Bishop
+// This file Copyright 2008-2020 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -67,7 +67,7 @@ for(var marker=1;marker<=mapprops.maxmarkers;marker++)
 
 var legal={"^lon"             : "^[-0-9.]+$",
            "^lat"             : "^[-0-9.]+$",
-           "^zoom"            : "^[0-9]+$",
+           "^zoom"            : "^[-0-9.]+$",
 
            "^lon[1-9]"        : "^[-0-9.]+$",
            "^lat[1-9]"        : "^[-0-9.]+$",
@@ -165,6 +165,12 @@ function form_init()            // called from router.html
  // Fill in the waypoints
 
  vismarkers=0;
+ urlmarkers=0;
+
+ minlat=90;
+ maxlat=-90;
+ minlon=180;
+ maxlon=-180;
 
  for(var marker=mapprops.maxmarkers;marker>=1;marker--)
    {
@@ -179,11 +185,21 @@ function form_init()            // called from router.html
        formSetSearch(marker,search);
        formSetCoords(marker,lon,lat);
 
+       lat=Number(lat);
+       lon=Number(lon);
+
+       if(lat<minlat) minlat=lat;
+       if(lat>maxlat) maxlat=lat;
+
+       if(lon<minlon) minlon=lon;
+       if(lon>maxlon) maxlon=lon;
+
        markerAddMap(marker);
 
        markerSearch(marker);
 
        vismarkers++;
+       urlmarkers++;
       }
     else if(lon !== undefined && lat !== undefined && lon !== "" && lat !== "")
       {
@@ -191,11 +207,21 @@ function form_init()            // called from router.html
 
        formSetCoords(marker,lon,lat);
 
+       lat=Number(lat);
+       lon=Number(lon);
+
+       if(lat<minlat) minlat=lat;
+       if(lat>maxlat) maxlat=lat;
+
+       if(lon<minlon) minlon=lon;
+       if(lon>maxlon) maxlon=lon;
+
        markerAddMap(marker);
 
        markerCoords(marker);
 
        vismarkers++;
+       urlmarkers++;
       }
     else if(search !== undefined && search !== "")
       {
@@ -234,6 +260,26 @@ function form_init()            // called from router.html
  else
     formSetLoopReverse("reverse",false);
 
+ // Zoom the map
+
+ if(urlmarkers>1)
+   {
+    var lon =args["lon"];
+    var lat =args["lat"];
+    var zoom=args["zoom"];
+
+    if(lon === undefined || lat === undefined || zoom === undefined)
+      {
+       var deltalon = 0.025 * ( maxlon - minlon );
+       var deltalat = 0.025 * ( maxlat - minlat );
+
+       var markerextent=ol.extent.boundingExtent([ol.proj.fromLonLat([minlon-deltalon,minlat-deltalat]),
+                                                  ol.proj.fromLonLat([maxlon+deltalon,maxlat+deltalat])]);
+
+       map.getView().fit(markerextent,map.getSize());
+      }
+   }
+
  // Update the transport type with the URL settings which updates all HTML forms to defaults.
 
  var transport=routino.transport;
@@ -281,7 +327,7 @@ function form_init()            // called from router.html
  var cookies=document.cookie.split("; ");
 
  for(var cookie=0;cookie<cookies.length;cookie++)
-    if(cookies[cookie].substr(0,"Routino-home".length)=="Routino-home")
+    if(cookies[cookie].startsWith("Routino-home"))
       {
        var data=cookies[cookie].split(/[=:;]/);
 
@@ -300,7 +346,7 @@ function form_init()            // called from router.html
        markerMoveHome(1);
    }
 
- updateURLs();
+ updateURLs(true);
 
  updateSearchButtons();
 }
@@ -344,7 +390,7 @@ function formSetLanguage(value) // called from router.html (with no arguments)
     routino.language=value;
    }
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -378,7 +424,7 @@ function formSetTransport(value) // called from router.html
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -411,7 +457,7 @@ function formSetHighway(type,value) // called from router.html (with one argumen
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -448,7 +494,7 @@ function formSetSpeed(type,value) // called from router.html (with one argument)
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -483,7 +529,7 @@ function formSetProperty(type,value) // called from router.html (with one argume
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -538,7 +584,7 @@ function formSetRestriction(type,value) // called from router.html (with one arg
 
  paramschanged=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -564,7 +610,7 @@ function formSetCoords(marker,lon,lat) // called from router.html (with one argu
     routino.point[marker].lon="";
     routino.point[marker].lat="";
 
-    updateURLs();
+    updateURLs(true);
    }
  else
    {
@@ -651,7 +697,7 @@ function formSetLoopReverse(type,value) // called from router.html (with one arg
  else
     routino.reverse=value;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -735,6 +781,9 @@ function buildMapArguments()
 
  var zoom = map.getView().getZoom();
 
+ if( ! Number.isInteger(zoom) )
+    zoom = format5f(zoom);
+
  return "lat=" + format5f(lonlat[1]) + ";lon=" + format5f(lonlat[0]) + ";zoom=" + zoom;
 }
 
@@ -743,11 +792,15 @@ function buildMapArguments()
 // Update the URLs
 //
 
-function updateURLs()
+function updateURLs(addhistory)
 {
- var urlargs1=buildURLArguments(true);
- var urlargs2=buildURLArguments(false);
- var mapargs=buildMapArguments();
+ var urlargs =buildURLArguments(false);
+ var mapargs =buildMapArguments();
+ var langargs="language=" + routino.language;
+ var libargs =";library=" + mapprops.library;
+
+ if(!mapprops.libraries)
+    libargs="";
 
  var links=document.getElementsByTagName("a");
 
@@ -756,17 +809,23 @@ function updateURLs()
     var element=links[i];
 
     if(element.id == "permalink_url")
-       element.href=location.pathname + "?" + urlargs1 + ";" + mapargs;
+       element.href=location.pathname + "?" + urlargs + ";" + langargs + ";" + mapargs + libargs;
 
     if(element.id == "visualiser_url")
-       element.href="visualiser.html" + "?" + mapargs;
+       if(location.pathname.match(/router\.html\.([a-zA-Z-]+)$/))
+          element.href="visualiser.html." + RegExp.$1 + "?" + mapargs + libargs;
+       else
+          element.href="visualiser.html" + "?" + mapargs + libargs;
 
     if(element.id == "edit_url")
        element.href=mapprops.editurl + "?" + mapargs;
 
     if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
-       element.href="router.html" + "." + RegExp.$1 + "?" + urlargs2 + ";" + mapargs;
+       element.href="router.html" + "." + RegExp.$1 + "?" + urlargs + ";" + mapargs + libargs;
    }
+
+ if(addhistory)
+    history.replaceState(null, null, location.pathname + "?" + urlargs + ";" + langargs + libargs);
 }
 
 
@@ -798,7 +857,7 @@ function map_init()             // called from router.html
 
  // Add map tile layers
 
- for(var l=0; l<mapprops.mapdata.length; l++)
+ for(var l=mapprops.mapdata.length-1; l>=0; l--)
    {
     mapprops.mapdata[l].tiles.url=mapprops.mapdata[l].tiles.url.replace(/\$\{/g,"{");
     var urls;
@@ -858,7 +917,7 @@ function map_init()             // called from router.html
  gpx_style={shortest: new ol.style.Style({stroke: new ol.style.Stroke({width: 3, color: "#00FF00"})}),
             quickest: new ol.style.Style({stroke: new ol.style.Stroke({width: 3, color: "#0000FF"})})};
 
- // Add a vectors layer
+ // Add a markers vectors layer
 
  layerVectors = new ol.layer.Vector({source: new ol.source.Vector()});
  map.addLayer(layerVectors);
@@ -909,7 +968,7 @@ function map_init()             // called from router.html
 
  // Move the map
 
- map.on("moveend", updateURLs, map);
+ map.on("moveend", (function() { updateURLs(false);}), map);
 
  var lon =args["lon"];
  var lat =args["lat"];
@@ -917,6 +976,10 @@ function map_init()             // called from router.html
 
  if(lon !== undefined && lat !== undefined && zoom !== undefined)
    {
+    lat  = Number(lat);
+    lon  = Number(lon);
+    zoom = Number(zoom);
+
     if(lon<mapprops.westedge) lon=mapprops.westedge;
     if(lon>mapprops.eastedge) lon=mapprops.eastedge;
 
@@ -926,7 +989,7 @@ function map_init()             // called from router.html
     if(zoom<mapprops.zoomout) zoom=mapprops.zoomout;
     if(zoom>mapprops.zoomin)  zoom=mapprops.zoomin;
 
-    map.getView().setCenter(ol.proj.fromLonLat([Number(lon),Number(lat)]));
+    map.getView().setCenter(ol.proj.fromLonLat([lon,lat]));
     map.getView().setZoom(zoom);
    }
  else
@@ -942,7 +1005,7 @@ function map_init()             // called from router.html
     edit_url.href=mapprops.editurl;
    }
 
- updateURLs();
+ updateURLs(false);
 }
 
 
@@ -968,7 +1031,7 @@ function dragMarkerComplete(feature)
     if(feature==markers[marker])
        dragMarkerSetForm(marker);
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -1208,7 +1271,7 @@ function markerAddMap(marker)
 
  markersmoved=true;
 
- updateURLs();
+ updateURLs(true);
 
  updateSearchButtons();
 }
@@ -1230,7 +1293,7 @@ function markerRemoveMap(marker)
 
  markersmoved=true;
 
- updateURLs();
+ updateURLs(true);
 
  updateSearchButtons();
 }
@@ -1508,14 +1571,14 @@ function markerSetClearHome(marker,home)
     homelat=null;
     homelon=null;
 
-    cookie="Routino-home=unset";
+    cookie="Routino-home=";
 
     date.setUTCFullYear(date.getUTCFullYear()-1);
 
     routino.point[marker].home=false;
    }
 
- document.cookie=cookie + ";expires=" + date.toGMTString();
+ document.cookie=cookie + ";Expires=" + date.toGMTString() + ";SameSite=Strict";
 
  updateIcon(marker);
 
@@ -1639,7 +1702,7 @@ function markersReverse()       // called from router.html
 
  markersmoved=true;
 
- updateURLs();
+ updateURLs(true);
 }
 
 
@@ -1670,7 +1733,7 @@ function markersLoop()          // called from router.html
 
  markersmoved=true;
 
- updateURLs();
+ updateURLs(true);
 
  updateSearchButtons();
 }
