@@ -3,7 +3,7 @@
 #
 # Part of the Routino routing software.
 #
-# This file Copyright 2012-2014 Andrew M. Bishop
+# This file Copyright 2012-2014, 2016 Andrew M. Bishop
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -22,7 +22,7 @@
 use strict;
 
 # Use the directory paths script
-require "paths.pl";
+require "./paths.pl";
 
 # Use the perl encoding/decoding functions
 use Encode qw(decode encode);
