@@ -83,7 +83,7 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <td><a id="lang_en_url" href="visualiser.html.en" title="English language webpage">English</a>
 <td>(EN)
 <tr>
-<td><a id="lang_cs_url" href="visualiser.html.cs" title="Česky webpage">Česky</a>
+<td><a id="lang_cs_url" href="visualiser.html.cs" title="Stránka v češtině">Česky</a>
 <td>(CS)
 <tr>
 <td><a id="lang_de_url" href="visualiser.html.de" title="Deutsche Webseite">Deutsch</a>
@@ -92,6 +92,9 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <td><a id="lang_es_url" href="visualiser.html.es" title="Español webpage">Español</a>
 <td>(ES)
 <tr>
+<td><a id="lang_fi_url" href="visualiser.html.fi" title="Suomi webpage">Suomi</a>
+<td>(FI)
+<tr>
 <td><a id="lang_fr_url" href="visualiser.html.fr" title="Page web en français">Français</a>
 <td>(FR)
 <tr>
@@ -107,10 +110,7 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <td><a id="lang_pl_url" href="visualiser.html.pl" title="Polski webpage">Polski</a>
 <td>(PL)
 <tr>
-<td><a id="lang_pt_url" href="visualiser.html.pt" title="Português webpage">Português</a>
-<td>(PT)
-<tr>
-<td><a id="lang_ru_url" href="visualiser.html.ru" title="Страница на Русском языке">Русский</a>
+<td><a id="lang_ru_url" href="visualiser.html.ru" title="Страница на русском языке">Русский</a>
 <td>(RU)
 </table>
 <a target="translation" href="http://www.routino.org/translations/">Routino Translations</a>
@@ -235,8 +235,8 @@ Each segment of the chosen type of highway is drawn.
 <tr><td>Track: <td><input name="highway" type="radio" value="track" onchange="displayData('highway');">
 <tr><td>Cycleway: <td><input name="highway" type="radio" value="cycleway" onchange="displayData('highway');">
 <tr><td>Path: <td><input name="highway" type="radio" value="path" onchange="displayData('highway');">
-<tr><td>Steps: <td><input name="highway" type="radio" value="steps" onchange="displayData('highway');">
-<tr><td>Ferry: <td><input name="highway" type="radio" value="ferry" onchange="displayData('highway');">
+<tr><td>Schody: <td><input name="highway" type="radio" value="steps" onchange="displayData('highway');">
+<tr><td>Převoz: <td><input name="highway" type="radio" value="ferry" onchange="displayData('highway');">
 </table>
 </form>
 </div>
@@ -257,7 +257,7 @@ Each segment allowed for the chosen type of transport is drawn.
 <tr><td>Moped: <td><input name="transport" type="radio" value="moped" onchange="displayData('transport');">
 <tr><td>Motocykl:<td><input name="transport" type="radio" value="motorcycle" onchange="displayData('transport');">
 <tr><td>Motorcar: <td><input name="transport" type="radio" value="motorcar" onchange="displayData('transport');" checked>
-<tr><td>Goods: <td><input name="transport" type="radio" value="goods" onchange="displayData('transport');">
+<tr><td>Zboží: <td><input name="transport" type="radio" value="goods" onchange="displayData('transport');">
 <tr><td>HGV: <td><input name="transport" type="radio" value="hgv" onchange="displayData('transport');">
 <tr><td>PSV: <td><input name="transport" type="radio" value="psv" onchange="displayData('transport');">
 </table>
@@ -280,7 +280,7 @@ Each barrier blocking the chosen type of transport is drawn.
 <tr><td>Moped: <td><input name="barrier" type="radio" value="moped" onchange="displayData('barrier');">
 <tr><td>Motocykl:<td><input name="barrier" type="radio" value="motorcycle" onchange="displayData('barrier');">
 <tr><td>Motorcar: <td><input name="barrier" type="radio" value="motorcar" onchange="displayData('barrier');" checked>
-<tr><td>Goods: <td><input name="barrier" type="radio" value="goods" onchange="displayData('barrier');">
+<tr><td>Zboží: <td><input name="barrier" type="radio" value="goods" onchange="displayData('barrier');">
 <tr><td>HGV: <td><input name="barrier" type="radio" value="hgv" onchange="displayData('barrier');">
 <tr><td>PSV: <td><input name="barrier" type="radio" value="psv" onchange="displayData('barrier');">
 </table>
@@ -389,8 +389,8 @@ Each segment of the highways with a particular property is drawn.
 <tr><td>Multiple Lanes: <td><input name="property" type="radio" value="multilane" onchange="displayData('property');">
 <tr><td>Most: <td><input name="property" type="radio" value="bridge" onchange="displayData('property');">
 <tr><td>Tunel: <td><input name="property" type="radio" value="tunnel" onchange="displayData('property');">
-<tr><td>Walking Route: <td><input name="property" type="radio" value="footroute" onchange="displayData('property');">
-<tr><td>Bicycle Route: <td><input name="property" type="radio" value="bicycleroute" onchange="displayData('property');">
+<tr><td>Pěší trasa: <td><input name="property" type="radio" value="footroute" onchange="displayData('property');">
+<tr><td>Cyklotrasa: <td><input name="property" type="radio" value="bicycleroute" onchange="displayData('property');">
 </table>
 </form>
 </div>
@@ -453,7 +453,7 @@ To perform routing on the map use the link below.
 <div class="hideshow_box">
 <span class="hideshow_title">Routino statistiky</span>
 <div id="statistics_data"></div>
-<a id="statistics_link" href="statistics.cgi" onclick="displayStatistics();return(false);">Display data statistics</a>
+<a id="statistics_link" href="statistics.cgi" onclick="displayStatistics();return(false);">Zobrazit statistická data</a>
 </div>
 </div>
 
