@@ -83,7 +83,7 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <td><a id="lang_en_url" href="visualiser.html.en" title="English language webpage">English</a>
 <td>(EN)
 <tr>
-<td><a id="lang_cs_url" href="visualiser.html.cs" title="Česky webpage">Česky</a>
+<td><a id="lang_cs_url" href="visualiser.html.cs" title="Stránka v češtině">Česky</a>
 <td>(CS)
 <tr>
 <td><a id="lang_de_url" href="visualiser.html.de" title="Deutsche Webseite">Deutsch</a>
@@ -92,6 +92,9 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <td><a id="lang_es_url" href="visualiser.html.es" title="Español webpage">Español</a>
 <td>(ES)
 <tr>
+<td><a id="lang_fi_url" href="visualiser.html.fi" title="Suomi webpage">Suomi</a>
+<td>(FI)
+<tr>
 <td><a id="lang_fr_url" href="visualiser.html.fr" title="Page web en français">Français</a>
 <td>(FR)
 <tr>
@@ -107,10 +110,7 @@ Only data relevant for routing is displayed and some will therefore be excluded.
 <td><a id="lang_pl_url" href="visualiser.html.pl" title="Polski webpage">Polski</a>
 <td>(PL)
 <tr>
-<td><a id="lang_pt_url" href="visualiser.html.pt" title="Português webpage">Português</a>
-<td>(PT)
-<tr>
-<td><a id="lang_ru_url" href="visualiser.html.ru" title="Страница на Русском языке">Русский</a>
+<td><a id="lang_ru_url" href="visualiser.html.ru" title="Страница на русском языке">Русский</a>
 <td>(RU)
 </table>
 <a target="translation" href="http://www.routino.org/translations/">Routino Translations</a>
