@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2014, 2019 Andrew M. Bishop
+// This file Copyright 2008-2014, 2019, 2020 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -48,9 +48,11 @@ var data_types=[
 
 // Process the URL query string and extract the arguments
 
-var legal={"^lon"  : "^[-0-9.]+$",
-           "^lat"  : "^[-0-9.]+$",
-           "^zoom" : "^[0-9]+$"};
+var legal={"^lon"     : "^[-0-9.]+$",
+           "^lat"     : "^[-0-9.]+$",
+           "^zoom"    : "^[-0-9.]+$",
+           "^data"    : "^.+$",
+           "^subdata" : "^.+$"};
 
 var args={};
 
@@ -83,8 +85,9 @@ if(location.search.length>1)
 ////////////////////////////////////////////////////////////////////////////////
 
 var map;
-var layerMap=[], layerHighlights, layerVectors, layerBoxes;
-
+var layerMap=[], layerHighlights, layerVectors;
+var vectorData=[];
+var displaytype="", displaysubtype="";
 
 //
 // Initialise the 'map' object
@@ -106,7 +109,7 @@ function map_init()             // called from visualiser.html
 
  // Add map tile layers
 
- for(var l=0; l<mapprops.mapdata.length; l++)
+ for(var l=mapprops.mapdata.length-1; l>=0; l--)
    {
     mapprops.mapdata[l].tiles.url=mapprops.mapdata[l].tiles.url.replace(/\$\{/g,"{");
     var urls;
@@ -175,15 +178,9 @@ function map_init()             // called from visualiser.html
 
  createPopup();
 
- // Add a boxes layer
-
- layerBoxes = new ol.layer.Vector({source: new ol.source.Vector()});
-
- map.addLayer(layerBoxes);
-
  // Move the map
 
- map.on("moveend", updateURLs, map);
+ map.on("moveend", (function() { displayMoreData(); updateURLs(false);}), map);
 
  var lon =args["lon"];
  var lat =args["lat"];
@@ -191,6 +188,10 @@ function map_init()             // called from visualiser.html
 
  if(lon !== undefined && lat !== undefined && zoom !== undefined)
    {
+    lat  = Number(lat);
+    lon  = Number(lon);
+    zoom = Number(zoom);
+
     if(lon<mapprops.westedge) lon=mapprops.westedge;
     if(lon>mapprops.eastedge) lon=mapprops.eastedge;
 
@@ -200,7 +201,7 @@ function map_init()             // called from visualiser.html
     if(zoom<mapprops.zoomout) zoom=mapprops.zoomout;
     if(zoom>mapprops.zoomin)  zoom=mapprops.zoomin;
 
-    map.getView().setCenter(ol.proj.fromLonLat([Number(lon),Number(lat)]));
+    map.getView().setCenter(ol.proj.fromLonLat([lon,lat]));
     map.getView().setZoom(zoom);
    }
  else
@@ -216,7 +217,17 @@ function map_init()             // called from visualiser.html
     edit_url.href=mapprops.editurl;
    }
 
- updateURLs();
+ // Select the data view if selected
+
+ var datatype=args["data"];
+ var datasubtype=args["subdata"];
+
+ if(datatype !== undefined)
+    displayData(datatype, datasubtype);
+
+ // Update the URL
+
+ updateURLs(false);
 }
 
 
@@ -254,6 +265,9 @@ function buildMapArguments()
 
  var zoom = map.getView().getZoom();
 
+ if( ! Number.isInteger(zoom) )
+    zoom = format5f(zoom);
+
  return "lat=" + format5f(lonlat[1]) + ";lon=" + format5f(lonlat[0]) + ";zoom=" + zoom;
 }
 
@@ -262,9 +276,19 @@ function buildMapArguments()
 // Update the URLs
 //
 
-function updateURLs()
+function updateURLs(addhistory)
 {
  var mapargs=buildMapArguments();
+ var dataargs=";data=" + displaytype;
+ var libargs=";library=" + mapprops.library;
+
+ if(displaytype === "")
+    dataargs="";
+ else if(displaysubtype !== "")
+    dataargs+=";subdata=" + displaysubtype;
+
+ if(!mapprops.libraries)
+    libargs="";
 
  var links=document.getElementsByTagName("a");
 
@@ -273,17 +297,23 @@ function updateURLs()
     var element=links[i];
 
     if(element.id == "permalink_url")
-       element.href=location.pathname + "?" + mapargs;
+       element.href=location.pathname + "?" + mapargs + dataargs + libargs;
 
     if(element.id == "router_url")
-       element.href="router.html" + "?" + mapargs;
+       if(location.pathname.match(/visualiser\.html\.([a-zA-Z-]+)$/))
+          element.href="router.html" + "." + RegExp.$1 + "?" + mapargs + libargs;
+       else
+          element.href="router.html" + "?" + mapargs + libargs;
 
     if(element.id == "edit_url")
        element.href=mapprops.editurl + "?" + mapargs;
 
     if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
-       element.href="visualiser.html" + "." + RegExp.$1 + "?" + mapargs;
+       element.href="visualiser.html" + "." + RegExp.$1 + "?" + mapargs + dataargs + libargs;
    }
+
+ if(addhistory)
+    history.replaceState(null, null, location.pathname + "?" + mapargs + dataargs + libargs);
 }
 
 
@@ -399,7 +429,7 @@ function runDumpSuccess(response)
 {
  var string=response.responseText;
 
- if(mapprops.editurl !== undefined && mapprops.editurl !== "")
+ if(mapprops.browseurl !== undefined && mapprops.browseurl !== "")
    {
     var types=["node", "way", "relation"];
     var Types=["Node", "Way", "Relation"];
@@ -511,22 +541,26 @@ function runStatisticsSuccess(response)
 // Get the requested data
 //
 
-function displayData(datatype)  // called from visualiser.html
+function displayData(datatype, datasubtype)  // called from visualiser.html
 {
+ // Display the form entry
+
  for(var data in data_types)
     hideshow_hide(data_types[data]);
 
  if(datatype !== "")
     hideshow_show(datatype);
 
+ if(datasubtype === undefined)
+    datasubtype="";
+
  // Delete the old data
 
+ vectorData=[];
+
  unselectFeature();
 
  layerVectors.getSource().clear();
- layerHighlights.getSource().clear();
-
- layerBoxes.getSource().clear();
 
  // Print the status
 
@@ -535,8 +569,84 @@ function displayData(datatype)  // called from visualiser.html
  // Return if just here to clear the data
 
  if(datatype === "")
+   {
+    displaytype = "";
+    displaysubtype = "";
+    updateURLs(true);
     return;
+   }
 
+ // Determine the type of data
+
+ switch(datatype)
+   {
+   case "junctions":
+    break;
+   case "super":
+    break;
+   case "waytype":
+    var waytypes=document.forms["waytypes"].elements["waytype"];
+    for(var w in waytypes)
+       if(datasubtype == waytypes[w].value)
+          waytypes[w].checked=true;
+       else if(waytypes[w].checked)
+          datasubtype=waytypes[w].value;
+    break;
+   case "highway":
+    var highways=document.forms["highways"].elements["highway"];
+    for(var h in highways)
+       if(datasubtype == highways[h].value)
+          highways[h].checked=true;
+       else if(highways[h].checked)
+          datasubtype=highways[h].value;
+    break;
+   case "transport":
+    var transports=document.forms["transports"].elements["transport"];
+    for(var t in transports)
+       if(datasubtype == transports[t].value)
+          transports[t].checked=true;
+       else if(transports[t].checked)
+          datasubtype=transports[t].value;
+    break;
+   case "barrier":
+    var transports=document.forms["barriers"].elements["barrier"];
+    for(var t in transports)
+       if(datasubtype == transports[t].value)
+          transports[t].checked=true;
+       else if(transports[t].checked)
+          datasubtype=transports[t].value;
+    break;
+   case "turns":
+    break;
+   case "speed":
+   case "weight":
+   case "height":
+   case "width":
+   case "length":
+    break;
+   case "property":
+    var properties=document.forms["properties"].elements["property"];
+    for(var p in properties)
+       if(datasubtype == properties[p].value)
+          properties[p].checked=true;
+       else if(properties[p].checked)
+          datasubtype=properties[p].value;
+    break;
+   case "errorlogs":
+    break;
+   }
+
+ // Update the URLs
+
+ displaytype = datatype;
+ displaysubtype = datasubtype;
+
+ displayMoreData();
+}
+
+
+function displayMoreData()
+{
  // Get the new data
 
  var mapbounds=map.getView().calculateExtent(map.getSize());
@@ -547,11 +657,11 @@ function displayData(datatype)  // called from visualiser.html
  url=url + ";latmin=" + format5f(ol.proj.toLonLat([mapbounds[0],mapbounds[1]])[1]);
  url=url + ";lonmax=" + format5f(ol.proj.toLonLat([mapbounds[2],mapbounds[3]])[0]);
  url=url + ";latmax=" + format5f(ol.proj.toLonLat([mapbounds[2],mapbounds[3]])[1]);
- url=url + ";data=" + datatype;
+ url=url + ";data=" + displaytype;
 
  // Use AJAX to get the data
 
- switch(datatype)
+ switch(displaytype)
    {
    case "junctions":
     ajaxGET(url, runJunctionsSuccess, runFailure);
@@ -560,39 +670,19 @@ function displayData(datatype)  // called from visualiser.html
     ajaxGET(url, runSuperSuccess, runFailure);
     break;
    case "waytype":
-    var waytype;
-    var waytypes=document.forms["waytypes"].elements["waytype"];
-    for(var h in waytypes)
-       if(waytypes[h].checked)
-          waytype=waytypes[h].value;
-    url+="-" + waytype;
+    url+="-" + displaysubtype;
     ajaxGET(url, runWaytypeSuccess, runFailure);
     break;
    case "highway":
-    var highway;
-    var highways=document.forms["highways"].elements["highway"];
-    for(var h in highways)
-       if(highways[h].checked)
-          highway=highways[h].value;
-    url+="-" + highway;
+    url+="-" + displaysubtype;
     ajaxGET(url, runHighwaySuccess, runFailure);
     break;
    case "transport":
-    var transport;
-    var transports=document.forms["transports"].elements["transport"];
-    for(var t in transports)
-       if(transports[t].checked)
-          transport=transports[t].value;
-    url+="-" + transport;
+    url+="-" + displaysubtype;
     ajaxGET(url, runTransportSuccess, runFailure);
     break;
    case "barrier":
-    var transport;
-    var transports=document.forms["barriers"].elements["barrier"];
-    for(var t in transports)
-       if(transports[t].checked)
-          transport=transports[t].value;
-    url+="-" + transport;
+    url+="-" + displaysubtype;
     ajaxGET(url, runBarrierSuccess, runFailure);
     break;
    case "turns":
@@ -606,47 +696,15 @@ function displayData(datatype)  // called from visualiser.html
     ajaxGET(url, runLimitSuccess, runFailure);
     break;
    case "property":
-    var property;
-    var properties=document.forms["properties"].elements["property"];
-    for(var p in properties)
-       if(properties[p].checked)
-          property=properties[p].value;
-    url+="-" + property;
+    url+="-" + displaysubtype;
     ajaxGET(url, runPropertySuccess, runFailure);
     break;
    case "errorlogs":
     ajaxGET(url, runErrorlogSuccess, runFailure);
     break;
    }
-}
 
-
-//
-// Add a bounding box
-//
-
-function addBox(words)
-{
- var lat1=Number(words[0]);
- var lon1=Number(words[1]);
- var lat2=Number(words[2]);
- var lon2=Number(words[3]);
-
- var box = new ol.Feature();
-
- var geometry = new ol.geom.LineString([ol.proj.fromLonLat([lon1,lat1]),
-                                        ol.proj.fromLonLat([lon1,lat2]),
-                                        ol.proj.fromLonLat([lon2,lat2]),
-                                        ol.proj.fromLonLat([lon2,lat1]),
-                                        ol.proj.fromLonLat([lon1,lat1])]);
-
- var style = new ol.style.Style({stroke: new ol.style.Stroke({width: 2, color: "red"})});
-
- box.setGeometry(geometry);
-
- box.setStyle(style);
-
- layerBoxes.getSource().addFeature(box);
+ updateURLs(true);
 }
 
 
@@ -684,10 +742,16 @@ function runJunctionsSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat=Number(words[1]);
        var lon=Number(words[2]);
        var count=words[3];
@@ -704,7 +768,7 @@ function runJunctionsSuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","junctions",lines.length-2);
+ displayStatus("data","junctions",Object.keys(vectorData).length);
 }
 
 
@@ -733,10 +797,16 @@ function runSuperSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat=Number(words[1]);
        var lon=Number(words[2]);
 
@@ -767,7 +837,7 @@ function runSuperSuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","super",lines.length-2);
+ displayStatus("data","super",Object.keys(vectorData).length);
 }
 
 
@@ -789,10 +859,16 @@ function runWaytypeSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat1=Number(words[1]);
        var lon1=Number(words[2]);
        var lat2=Number(words[3]);
@@ -828,7 +904,7 @@ function runWaytypeSuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","waytype",lines.length-2);
+ displayStatus("data","waytype",Object.keys(vectorData).length);
 }
 
 
@@ -849,10 +925,16 @@ function runHighwaySuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat1=Number(words[1]);
        var lon1=Number(words[2]);
        var lat2=Number(words[3]);
@@ -873,7 +955,7 @@ function runHighwaySuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","highway",lines.length-2);
+ displayStatus("data","highway",Object.keys(vectorData).length);
 }
 
 
@@ -894,10 +976,16 @@ function runTransportSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat1=Number(words[1]);
        var lon1=Number(words[2]);
        var lat2=Number(words[3]);
@@ -918,7 +1006,7 @@ function runTransportSuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","transport",lines.length-2);
+ displayStatus("data","transport",Object.keys(vectorData).length);
 }
 
 
@@ -940,10 +1028,16 @@ function runBarrierSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat=Number(words[1]);
        var lon=Number(words[2]);
 
@@ -959,7 +1053,7 @@ function runBarrierSuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","barrier",lines.length-2);
+ displayStatus("data","barrier",Object.keys(vectorData).length);
 }
 
 
@@ -980,10 +1074,16 @@ function runTurnsSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat1=Number(words[1]);
        var lon1=Number(words[2]);
        var lat2=Number(words[3]);
@@ -1007,7 +1107,7 @@ function runTurnsSuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","turns",lines.length-2);
+ displayStatus("data","turns",Object.keys(vectorData).length);
 }
 
 
@@ -1033,10 +1133,16 @@ function runLimitSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat=Number(words[1]);
        var lon=Number(words[2]);
        var number=words[3];
@@ -1085,7 +1191,7 @@ function runLimitSuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","limit",lines.length-2);
+ displayStatus("data","limit",Object.keys(vectorData).length);
 }
 
 
@@ -1106,10 +1212,16 @@ function runPropertySuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat1=Number(words[1]);
        var lon1=Number(words[2]);
        var lat2=Number(words[3]);
@@ -1130,7 +1242,7 @@ function runPropertySuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","property",lines.length-2);
+ displayStatus("data","property",Object.keys(vectorData).length);
 }
 
 
@@ -1152,10 +1264,16 @@ function runErrorlogSuccess(response)
     var words=lines[line].split(" ");
 
     if(line === 0)
-       addBox(words);
+       continue;
     else if(words[0] !== "")
       {
        var dump=words[0];
+
+       if(vectorData[dump])
+          continue;
+       else
+          vectorData[dump]=1;
+
        var lat=Number(words[1]);
        var lon=Number(words[2]);
 
@@ -1171,7 +1289,7 @@ function runErrorlogSuccess(response)
 
  layerVectors.getSource().addFeatures(features);
 
- displayStatus("data","errorlogs",lines.length-2);
+ displayStatus("data","errorlogs",Object.keys(vectorData).length);
 }
 
 
